class MyadventistUser
  include ActiveModel::Model

  validates :first_name, :last_name, :email, :password, presence: true
  validates :email, email: true
  # Password complexity: at least 1 uppercase and lowercase character
  validates :password, length: {minimum: 12}, format: { with: /(?=.*[A-Z])(?=.*[a-z]).*/, message: I18n.t("dash_myadventist.myadventist_user.validations.password_complexity") }

  attr_accessor :first_name, :last_name, :email, :password, :adventist_uid

  def save
    return false unless valid?
    create_account
  end

  def attributes
    {
      first_name: first_name,
      last_name: last_name,
      email: email,
      password: password,
      verified: 1 #hmmm
    }
  end

  def create_account
    myadventist = MyadventistApi.new
    response = myadventist.create_account(attributes)
    if response.success?
      self.adventist_uid = response.data[:user_id]
      return true
    else
      map_myadventist_errors(response)
    end
    false
  end

  private

  # map myadventist errors to attributes
  def map_myadventist_errors(response)
    if response.data[:error_description].include?("Email is already in use")
      errors.add :email, "already taken"
    else
      errors.add :base, response.data[:error_description]
    end
  end

end